@@ -65,7 +65,7 @@ local function add_files_to_server(refresh, rockfiles, server, upload_server)
    fs.change_dir(local_cache)
 
    util.printout("Updating manifest...")
-   manif.make_manifest(local_cache)
+   manif.make_manifest(local_cache, "one")
    util.printout("Updating index.html...")
    index.make_index(local_cache)
 
