@@ -26,15 +26,12 @@ function dir_name(pathname)
    return (pathname:gsub("/*$", ""):match("(.*/)[^/]*")) or ""
 end
 
-function strip_base_dir(pathname)
-   return pathname:gsub("^[^/]*/", "")
-end
-
 --- Describe a path in a cross-platform way.
 -- Use this function to avoid platform-specific directory
--- separators in other modules. If the first item contains a 
--- protocol descriptor (e.g. "http:"), paths are always constituted
--- with forward slashes.
+-- separators in other modules. Removes trailing slashes from
+-- each component given, to avoid repeated separators.
+-- Separators inside strings are kept, to handle URLs containing
+-- protocols.
 -- @param ... strings representing directories
 -- @return string: a string with a platform-specific representation
 -- of the path.
@@ -42,14 +39,14 @@ function path(...)
    local items = {...}
    local i = 1
    while items[i] do
-      items[i] = items[i]:gsub("/*$", "")
+      items[i] = items[i]:gsub("(.+)/+$", "%1")
       if items[i] == "" then
          table.remove(items, i)
       else
          i = i + 1
       end
    end
-   return table.concat(items, "/")
+   return (table.concat(items, "/"):gsub("(.+)/+$", "%1"))
 end
 
 --- Split protocol and path from an URL or local pathname.
@@ -67,3 +64,7 @@ function split_url(url)
    end
    return protocol, pathname
 end
+
+function normalize(name)
+   return name:gsub("\\", "/"):gsub("(.)/*$", "%1")
+end
