@@ -14,7 +14,7 @@ local search = require("luarocks.search")
 local util = require("luarocks.util")
 local cfg = require("luarocks.cfg")
 local path = require("luarocks.path")
-local rep = require("luarocks.rep")
+local repos = require("luarocks.repos")
 local deps = require("luarocks.deps")
 
 rock_manifest_cache = {}
@@ -31,7 +31,11 @@ local function save_table(where, name, tbl)
    assert(type(tbl) == "table")
 
    local filename = dir.path(where, name)
-   return persist.save_from_table(filename, tbl)
+   local ok, err = persist.save_from_table(filename..".tmp", tbl)
+   if ok then
+      ok, err = fs.replace_file(filename, filename..".tmp")
+   end
+   return ok, err
 end
 
 function load_rock_manifest(name, version, root)
@@ -181,14 +185,17 @@ end
 -- and any dependency inconsistencies or missing dependencies are reported to
 -- standard error.
 -- @param manifest table: a manifest table.
-local function update_dependencies(manifest)
+local function update_dependencies(manifest, deps_mode)
+   assert(type(manifest) == "table")
+   assert(type(deps_mode) == "string")
+
    for pkg, versions in pairs(manifest.repository) do
-      for version, repos in pairs(versions) do
+      for version, repositories in pairs(versions) do
          local current = pkg.." "..version
-         for _, repo in ipairs(repos) do
+         for _, repo in ipairs(repositories) do
             if repo.arch == "installed" then
                local missing
-               repo.dependencies, missing = deps.scan_deps({}, {}, manifest, pkg, version)
+               repo.dependencies, missing = deps.scan_deps({}, {}, manifest, pkg, version, deps_mode)
                repo.dependencies[pkg] = nil
                if missing then
                   for miss, err in pairs(missing) do
@@ -210,9 +217,10 @@ end
 -- @param manifest table: A manifest table (must contain repository, modules, commands tables).
 -- It will be altered to include the search results.
 -- @return boolean or (nil, string): true in case of success, or nil followed by an error message.
-local function store_results(results, manifest)
+local function store_results(results, manifest, deps_mode)
    assert(type(results) == "table")
    assert(type(manifest) == "table")
+   assert(type(deps_mode) == "string")
 
    for name, versions in pairs(results) do
       local pkgtable = manifest.repository[name] or {}
@@ -226,8 +234,8 @@ local function store_results(results, manifest)
                if not rock_manifest then
                   return nil, "rock_manifest file not found for "..name.." "..version.." - not a LuaRocks 2 tree?"
                end
-               entrytable.modules = store_package_items(rep.package_modules, name, version, manifest.modules)
-               entrytable.commands = store_package_items(rep.package_commands, name, version, manifest.commands)
+               entrytable.modules = store_package_items(repos.package_modules, name, version, manifest.modules)
+               entrytable.commands = store_package_items(repos.package_commands, name, version, manifest.commands)
             end
             table.insert(versiontable, entrytable)
          end
@@ -235,7 +243,7 @@ local function store_results(results, manifest)
       end
       manifest.repository[name] = pkgtable
    end
-   update_dependencies(manifest)
+   update_dependencies(manifest, deps_mode)
    sort_package_matching_table(manifest.modules)
    sort_package_matching_table(manifest.commands)
    return true
@@ -247,8 +255,11 @@ end
 -- @param repo A local repository directory.
 -- @return boolean or (nil, string): True if manifest was generated,
 -- or nil and an error message.
-function make_manifest(repo)
+function make_manifest(repo, deps_mode)
    assert(type(repo) == "string")
+   assert(type(deps_mode) == "string")
+
+   if deps_mode == "none" then deps_mode = cfg.deps_mode end
 
    if not fs.is_dir(repo) then
       return nil, "Cannot access repository at "..repo
@@ -261,7 +272,7 @@ function make_manifest(repo)
    local manifest = { repository = {}, modules = {}, commands = {} }
    manif_core.manifest_cache[repo] = manifest
 
-   local ok, err = store_results(results, manifest)
+   local ok, err = store_results(results, manifest, deps_mode)
    if not ok then return nil, err end
 
    return save_table(repo, "manifest", manifest)
@@ -277,17 +288,20 @@ end
 -- the default local repository configured as cfg.rocks_dir is used.
 -- @return boolean or (nil, string): True if manifest was generated,
 -- or nil and an error message.
-function update_manifest(name, version, repo)
+function update_manifest(name, version, repo, deps_mode)
    assert(type(name) == "string")
    assert(type(version) == "string")
    repo = path.rocks_dir(repo or cfg.root_dir)
+   assert(type(deps_mode) == "string")
+   
+   if deps_mode == "none" then deps_mode = cfg.deps_mode end
 
    util.printout("Updating manifest for "..repo)
 
    local manifest, err = load_manifest(repo)
    if not manifest then
       util.printerr("No existing manifest. Attempting to rebuild...")
-      local ok, err = make_manifest(repo)
+      local ok, err = make_manifest(repo, deps_mode)
       if not ok then
          return nil, err
       end
@@ -299,7 +313,7 @@ function update_manifest(name, version, repo)
 
    local results = {[name] = {[version] = {{arch = "installed", repo = repo}}}}
 
-   local ok, err = store_results(results, manifest)
+   local ok, err = store_results(results, manifest, deps_mode)
    if not ok then return nil, err end
 
    return save_table(repo, "manifest", manifest)
